#!/usr/bin/env ruby

# encoding settings
$nrof_subbands = 8
$nrof_blocks = 16
$sbc_join = 1
$bitpool = 255

if not ARGV[0]
    printf("Please provide a source audio file in the command line\n")
    exit(1)
end

# FIXME: properly escape file name
def get_file_info(filename)
    x = `sox -V #{filename} -n 2>&1`
    channels = nil
    if x =~ /^Channels       : (\d+)$/ then
        channels = $1.to_i
    end
    samplerate = nil
    if x =~ /^Sample Rate    : (\d+)$/ then
        samplerate = $1.to_i
    end
    if not channels then error("sss") end
    return [channels, samplerate]
end

def get_recommended_hq_encode_cmdline(filename)
    fileinfo = get_file_info(filename)
    p fileinfo
    nrof_channels = fileinfo[0]
    fs = fileinfo[1]

    bitpool_limit = nrof_channels * 16 * $nrof_subbands
    bitpool_limit = 255 if bitpool_limit > 255
    $bitpool = bitpool_limit if $bitpool > bitpool_limit

    frame_length = 4 + (4 * $nrof_subbands * nrof_channels) / 8
    if nrof_channels == 1 then
        frame_length += $nrof_blocks * nrof_channels * $bitpool / 8
    else
        frame_length += ($sbc_join * $nrof_subbands + $nrof_blocks * $bitpool) / 8
    end
    bit_rate = 8 * frame_length * fs / $nrof_subbands / $nrof_blocks
    if nrof_channels == 1 then
        return ["-S -s#{$nrof_subbands} -B#{$nrof_blocks} -b#{$bitpool}",
                "-l#{$nrof_blocks} -n#{$nrof_subbands} -r#{bit_rate}"]
    elsif $sbc_join == 1 then
        return ["-j -S -s#{$nrof_subbands} -B#{$nrof_blocks} -b#{$bitpool}",
                "-j -l#{$nrof_blocks} -n#{$nrof_subbands} -r#{bit_rate}"]
    else
        return ["-S -s#{$nrof_subbands} -B#{$nrof_blocks} -b#{$bitpool}",
                "-l#{$nrof_blocks} -n#{$nrof_subbands} -r#{bit_rate}"]
    end
end

x = get_recommended_hq_encode_cmdline(ARGV[0])
p x
`sox #{ARGV[0]} --endian big -s -2 -t au tmp.au`
`sox #{ARGV[0]} --endian little -s -2 -t wav tmp.wav`
`./sbcenc #{x[0]} tmp.au > tmp_sbcenc.sbc`
`wine ./sbc_encoder.exe #{x[1]} -otmp_ref.sbc tmp.wav 2> /dev/null`

`./sbcdec -f tmp_sbcdec.au tmp_sbcenc.sbc`
`sox tmp_sbcdec.au --endian little -s -2 -t wav tmp_sbcdec.wav`
`wine ./sbc_decoder.exe -otmp_ref.wav tmp_ref.sbc 2> /dev/null`
`wine ./sbc_decoder.exe -otmp_sbcenc.wav tmp_sbcenc.sbc  2> /dev/null`
printf("--- comparing original / sbcenc + sbcdec ---\n%s\n", `./tiny_psnr tmp_sbcdec.wav tmp.wav 2 - 44 | grep stddev`)
printf("--- comparing original / sbcenc + sbc_decoder.exe ---\n%s\n", `./tiny_psnr tmp_sbcenc.wav tmp.wav 2 - 44 | grep stddev`)
printf("--- comparing original / sbc_encoder.exe + sbc_decoder.exe ---\n%s\n", `./tiny_psnr tmp_ref.wav tmp.wav 2 - 44 | grep stddev`)
printf("--- comparing sbcenc + sbc_decoder.exe / sbc_encoder.exe + sbc_decoder.exe\n%s\n", `./tiny_psnr tmp_sbcenc.wav tmp_ref.wav 2 - 44 | grep stddev`)
